require "test/unit"
require "tempfile"
load "pfgray"

module Syslog
  def self.notice(*arg)
#    puts arg
  end
  def self.warning(*arg)
#    puts arg
  end
  def self.err(*arg)
#    puts arg
  end
end

class List
  attr_reader :value
end

class TC_List < Test::Unit::TestCase
  def setup()
  end
  def teardown()
  end

  def test_parse_hostname()
    l = List.new nil
    assert_equal(/^aaa\.bbb\.example\.com$/i, l.parse_hostname("aaa.bbb.example.com"))
  end

  def test_parse_hostname_begining_dot()
    l = List.new nil
    assert_equal(/\.example\.com$/i, l.parse_hostname(".example.com"))
  end

  def test_parse_hostname_wildcard()
    l = List.new nil
    assert_equal(/.*/, l.parse_hostname("*"))
  end

  def test_parse_hostname_re()
    l = List.new nil
    assert_equal(/abc/i, l.parse_hostname("/abc/"))
  end

  def test_parse_mailaddr()
    l = List.new nil
    assert_equal(/^aaa@example\.com$/i, l.parse_mailaddr("aaa@example.com"))
  end

  def test_parse_mailaddr_domain()
    l = List.new nil
    assert_equal(/@example\.com$/i, l.parse_mailaddr("example.com"))
  end

  def test_parse_mailaddr_subdomain()
    l = List.new nil
    assert_equal(/\.example\.com$/i, l.parse_mailaddr(".example.com"))
  end

  def test_parse_mailaddr_wildcard()
    l = List.new nil
    assert_equal(/.*/, l.parse_mailaddr("*"))
  end

  def test_parse_mailaddr_null()
    l = List.new nil
    assert_equal(/^$/, l.parse_mailaddr("<>"))
  end

  def test_read_file()
    tmpf = Tempfile.new("pfgray-test")
    tmpf.puts <<EOF
1.2.3.4\thelo.name\tsender@domain.name\trcpt@domain.name
*\t*\t*\t*
EOF
    tmpf.flush
    l = List.new(tmpf.path)
    assert_equal([[/^1\.2\.3\.4$/i,/^helo\.name$/i,/^sender@domain\.name$/i,/^rcpt@domain\.name$/i],[/.*/,/.*/,/.*/,/.*/]], l.value)
    tmpf.close!
  end

  def test_read_file_comment()
    tmpf = Tempfile.new("pfgray-test")
    tmpf.puts <<EOF
1.2.3.4\thelo.name\tsender@domain.name\trcpt@domain.name
# this is command line
EOF
    tmpf.flush
    l = List.new(tmpf.path)
    assert_equal([[/^1\.2\.3\.4$/i,/^helo\.name$/i,/^sender@domain\.name$/i,/^rcpt@domain\.name$/i]], l.value)
    tmpf.close!
  end

  def test_read_file_invalid()
    tmpf = Tempfile.new("pfgray-test")
    tmpf.puts <<EOF
1.2.3.4\thelo.name\tsender@domain.name\trcpt@domain.name
a\tb\tc
EOF
    tmpf.flush
    l = List.new(tmpf.path)
    assert_equal([[/^1\.2\.3\.4$/i,/^helo\.name$/i,/^sender@domain\.name$/i,/^rcpt@domain\.name$/i]], l.value)
    tmpf.close!
  end

  def test_check()
    tmpf = Tempfile.new("pfgray-test")
    tmpf.puts <<EOF
1.2.3.4\thelo.name\tsender@domain.name\trcpt@domain.name
EOF
    tmpf.flush
    l = List.new(tmpf.path)
    assert_equal(true, l.check("1.2.3.4", "unknown", "helo.name", "sender@domain.name", "rcpt@domain.name"))
    assert_equal(false, l.check("1.2.3.5", "unknown", "helo.name", "sender@domain.name", "rcpt@domain.name"))
    tmpf.close!
  end

  def test_parse_time()
    assert_equal(24, parse_time("24"))
  end

  def test_parse_time_s()
    assert_equal(24, parse_time("24s"))
  end

  def test_parse_time_m()
    assert_equal(24*60, parse_time("24m"))
  end

  def test_parse_time_h()
    assert_equal(24*3600, parse_time("24h"))
  end

  def test_parse_time_d()
    assert_equal(24*86400, parse_time("24d"))
  end
end
